/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*               Erase FLASH 2TS Memory on the MC68HC908AS60          	 */
/*                                                                	 */
/*************************************************************************/
/* File Name: Er_mrt.c				Copyright (c) Motorola 1999 	 */
/*                                                                  	 */
/* Current Revision: 1.0                                          	 */
/* Current Release Level: RP                                      	 */
/* Current Revision Release Date:  10/02/99                       	 */
/*                                                                	 */
/* Current Release Written By: Adeela Gill                        	 */
/*                             Motorola Systems Engineering       	 */
/*					 Austin, Texas					 */
/*                                                                	 */
/* Compiled Under: HiCross HC08 (HiWare)   Ver.: 5.0.5  		       */
/*                                                                	 */
/* Project Folder Name: FLASH_2TS                                 	 */
/*                                                                	 */
/* Part Family Software Routine Works With: HC08                         */
/* Part Module(s) Software Routine Works With: fls32k_a01                */
/*                                                                       */
/* Routine Size (Bytes):	346                                 	 */
/* Stack Space Used (Bytes):  9							 */
/* RAM Used (Bytes):		2							 */
/* Global Variables Used:	FLASH_addr                          	 */
/* Subroutine(s) Called:	EraseRoutine, WriteFLCR              	 */
/*                                                                	 */
/* Full Functional Description Of Routine Design:                 	 */
/*	Er_main.c is the main routine that demonstrates how to erase       */
/*    different size blocks of FLASH 2TS memory on the MC68HC908AS60.    */
/*										 		 */
/*************************************************************************/
/* Motorola reserves the right to make changes without further notice to */
/* any product herein. Motorola makes no warranty, representation or 	 */
/* guarantee regarding the suitability of its products for any particular*/
/* purpose, nor does Motorola assume any liability arising out of the    */
/* application or use of any product, circuit, and specifically disclaims*/
/* any and all liability, including without limitation consequential or	 */ 
/* incidental damages. "Typical" parameters can and do vary in different */
/* applications. All operating parameters, including "Typicals" must be  */
/* validated for each customer application by customer's technical 	 */
/* experts. Motorola does not convey any license under its patent rights */
/* nor the rights of others. Motorola products are not designed, 		 */
/* intended, or authorized for use as components in systems intended for */
/* surgical implant into the body, or other applications intended to 	 */
/* support or sustain life, or for any other application in which the    */
/* failure of the Motorola product could create a situation where 	 */
/* personal injury or death may occur. Should Buyer purchase or use    	 */
/* Motorola products for any such intended or unauthorized application,  */
/* Buyer shall indemnify and hold Motorola and its officers, employees,  */
/* subsidiaries, affiliates, and distributors harmless against all 	 */
/* claims, costs, damages, and expenses, and reasonable attorney fees 	 */
/* arising out of, directly or indirectly, any claim of personal injury  */
/* or death associated with such unintended or unauthorized use, even if */
/* such claim alleges that Motorola was negligent regarding the design or*/
/* manufacture of the part. Motorola and the Motorola symbol are 		 */
/* registered trademarks of Motorola, Inc.					 */
/*************************************************************************/

/*************************************************************************/
/*****                       Include Files		  	  	   *****/
/*************************************************************************/
#include <as60_flash_frk.c>		/* Equates for the registers and bits*/
						/*   of the HC908AS60 that are used	 */ 
#include <prog_er_var.c>		/* RAM variable definitions	 	 */

/*************************************************************************/
/*****                   Program-Specific Defines		  	   *****/
/*************************************************************************/
#define   eraseallrows	0x00		/* full array erase: 32K Bytes  	 */
						/*	cared addresses A15	       */
#define   erasehalfrows	0x10  	/* one-half array erase: 16K Bytes   */
						/*  	cared addresses A15-A14		 */
#define   erase8rows	0x20 		/* eight row erase:  512 Bytes 	 */
						/*  	cared addresses A15-A9	 	 */
#define   erase1row	0x30  	/* single row erase: 64 Bytes	 	 */
						/*  	cared addresses A15-A6	 	 */

/*************************************************************************/
/*****                     Function Definitions		  	         *****/
/*************************************************************************/
extern void		EraseRoutine	(void);
extern void 	writeFLCR 		(unsigned char);

/*************************************************************************/
/*****                       Erase Main Routine		  	  	   *****/
/*************************************************************************/
/* This routine initializes the 908AS60 before calling the erasing 	 */
/* routine, EraseRoutine. If a user plans to incorporate EraseRoutine  	 */
/* into his/her program, make sure the FLASH control registers and the 	 */
/* variable FLASH_addr are initialized before calling EraseRoutine. 	 */
/*												 */
/* After the initialization, EraseRoutine is called to erase a data 	 */
/* block of 64 bytes, 512 bytes, 16 Kbytes or 32bytes depending on the 	 */
/* value in the control registers.  FLASH_addr is any address within the */
/* block that will be erased. This program does not verify that the 	 */
/* operation was successful.								 */
/*************************************************************************/
void main ()
{					
	CONFIG1 = 0x71;			/*Turn off the COP, but leave the	 */
						/*  LVI on					 */

	FLCR1 = 0x00;			/*Set up the frequency divide  	 */   
	FLCR2 = 0x00;			/*  control bits in flcr1 & flcr2 	 */
						/*  for an appropriate charge pump   */
						/*  clock	 			 	 */

	FLASH_addr=(unsigned char *)0x4001;		
						/*Set address of erase block. This   */
						/*  is any address within the block  */
						/*  that will be erased			 */

	writeFLCR(eraseallrows);	/*Set BLK1 and BLK0 in appropriate	 */
						/*  FLASH control register to specify*/
						/*  the size of block to erase	 */

	EraseRoutine ();			/*Erase the block of FLASH including */
						/*  the specified address		 */
	
	while (1); 
							
}
 
/*************************************************************************/
/****				Subroutine Body Includes 			    ****/
/*************************************************************************/
#include	<prog_er_srt.c>		/*Prog8Bytes, WriteFLCR and Delay    */
						/*  Subroutines				 */

