/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*     Program / Margin Read and Erase FLASH 2TS Memory Subroutines      */
/*                                                                	 */
/*************************************************************************/
/* File Name: Prog_er_srt.c                 Copyright (c) Motorola 1999  */
/*                                                                  	 */
/* Current Revision: 1.0                                          	 */
/* Current Release Level: RP                                      	 */
/* Current Revision Release Date:  10/02/99                       	 */
/*                                                                	 */
/* Current Release Written By: Adeela Gill                        	 */
/*                             Motorola Systems Engineering       	 */
/*					 Austin, Texas					 */
/*                                                                	 */
/* Compiled Under: HiCross HC08 (HiWare)   Ver.: 5.0.5  		       */
/*                                                                	 */
/* Project Folder Name: FLASH_2TS                                 	 */
/*                                                                	 */
/* Part Family Software Routine Works With: HC08                         */
/* Part Module(s) Software Routine Works With: fls32k_a01                */
/*                                                                       */
/* Routine Sizes (Bytes):	composite with called routines:            */
/*                              Prog8Bytes    187 bytes                  */
/*					  EraseRoutine  75 bytes                   */
/*                              Delay         24 bytes                   */
/*                              WriteFLCR     30 bytes                   */
/* Stack Space Used (Bytes):  composite with called routines:            */
/*					  Prog8Bytes    8 bytes                    */
/*					  EraseRoutine  9 bytes                    */
/*                              Delay         2 bytes                    */
/*                              WriteFLCR     1 bytes                    */
/* RAM Used (Bytes):		composite with called routines:            */
/*					  Prog8Bytes    11 bytes                   */
/*					  EraseRoutine  2 bytes                    */
/*                              Delay         0 bytes                    */
/*                              WriteFLCR     2 bytes                    */
/*    											 */
/* Global Variable(s) Used:	Prog8Bytes    FLASH_addr, data, attempt    */
/*                            EraseRoutine  FLASH_addr                   */
/*                            Delay         None                         */
/*                         	WriteFLCR     FLASH_addr                   */
/*												 */
/* Submodule(s) Called:		Prog8Bytes	  WriteFLCR, Delay             */
/*					EraseRoutine  WriteFLCR, Delay             */
/*					Delay		  None                         */
/* 					WriteFLCR	  None              	 	 */
/*												 */
/* Calling Sequence:          EraseRoutine ()				       */
/*					int=Prog8Bytes ()	   	       		 */
/*					WriteFLCR (unsigned char)			 */
/*					Delay (unsigned char)				 */
/*												 */
/* Entry Label:			void EraseRoutine(),			 	 */
/*					unsigned char Prog8Bytes()            	 */
/*				  	void WriteFLCR (unsigned char)		 */
/*					void Delay (unsigned char)			 */
/*												 */
/* Entry Conditions:		EraseRoutine  2 byte address defined at    */
/*							    FLASH_addr                 */
/*					Prog8Bytes    2 bytes address defined at   */
/*							    FLASH_addr	             */
/* 							  8 programming bytes located  */
/*							    at variable data           */
/*					WriteFLCR 	  2 bytes address defined at   */
/*							    FLASH_addr			 */
/*							  1 byte FLCR bit definition   */
/*							    passed in		       */
/*					Delay		  1 byte delay value passed in */
/*												 */
/* Number of Exit Points:	4						  	 */
/*	Exit Label:			EraseRoutine  Erase990				 */
/*				  	Prog8Bytes    Prog990		  	 	 */
/*					WriteFLCR     FLCR990				 */
/*					Delay		  Delay990				 */
/*												 */
/* 	Exit Conditions:		EraseRoutine  None			  	 */
/*					Prog8Bytes	  pass/fail result returned    */
/*					WriteFLCR	  None				 */
/*					Delay		  None				 */
/*												 */
/* Full Functional Description Of Subroutine:                            */
/*   Prog_er_srt.c consists of two primary subroutines called	       */
/*   EraseRoutine and Prog8Bytes. These demonstrate 2TS flash erasing	 */
/*   and programming algorithms, respectively.  The routines also call 	 */
/*   other subroutines WriteFLCR and Delay.                              */
/*										 		 */
/*************************************************************************/
/* Motorola reserves the right to make changes without further notice to */
/* any product herein. Motorola makes no warranty, representation or 	 */
/* guarantee regarding the suitability of its products for any particular*/
/* purpose, nor does Motorola assume any liability arising out of the    */
/* application or use of any product, circuit, and specifically disclaims*/
/* any and all liability, including without limitation consequential or	 */ 
/* incidental damages. "Typical" parameters can and do vary in different */
/* applications. All operating parameters, including "Typicals" must be  */
/* validated for each customer application by customer's technical 	 */
/* experts. Motorola does not convey any license under its patent rights */
/* nor the rights of others. Motorola products are not designed, 		 */
/* intended, or authorized for use as components in systems intended for */
/* surgical implant into the body, or other applications intended to 	 */
/* support or sustain life, or for any other application in which the    */
/* failure of the Motorola product could create a situation where 	 */
/* personal injury or death may occur. Should Buyer purchase or use    	 */
/* Motorola products for any such intended or unauthorized application,  */
/* Buyer shall indemnify and hold Motorola and its officers, employees,  */
/* subsidiaries, affiliates, and distributors harmless against all 	 */
/* claims, costs, damages, and expenses, and reasonable attorney fees 	 */
/* arising out of, directly or indirectly, any claim of personal injury  */
/* or death associated with such unintended or unauthorized use, even if */
/* such claim alleges that Motorola was negligent regarding the design or*/
/* manufacture of the part. Motorola and the Motorola symbol are 		 */
/* registered trademarks of Motorola, Inc.					 */
/*************************************************************************/

/*************************************************************************/
/*****                     FLASH Delay Equates			         *****/
/*************************************************************************/
/* Each delay time related to FLASH 2TS program and erase operations is  */
/* calculated with bus speed 2.4576MHz.						 */
/*************************************************************************/
#define	tERASE	0xF0		/* FLASH 2TS block/bulk erase time	 */
						/*   (~101 ms) 				 */
#define	tKILL		0x0A		/* FLASH 2TS high-voltage kill time  */
						/*   (~210 us)				 */
#define	tHVD		0x02		/* FLASH 2TS return to read time	 */
						/*   (~51 us) 				 */
#define	tSTEP		0x32		/* FLASH 2TS page program step size	 */
						/*   (~1 ms) 				 */
#define	tHVTV		0x02		/* FLASH 2TS HVEN low to MARGIN high */
						/*   time (~51 us) 			 */
#define	tVTP		0x07		/* FLASH 2TS MARGIN high to PGM low  */
						/*   time (~151 us)  			 */
#define	flsPULSES	0x64		/* FLASH 2TS maximum page program 	 */
						/*   pulses (100 pulses)  		 */

/*************************************************************************/
/*****          Necessary Assembly-Level Commands Defines		   *****/
/*************************************************************************/
#define EnableInterrupts   {asm CLI;}
#define DisableInterrupts  {asm SEI;}

/*************************************************************************/
/*****                   Function Initializations           	   *****/
/*************************************************************************/
void			EraseRoutine	(void);
unsigned char	Prog8Bytes		(void);
void			writeFLCR		(unsigned char);
void			Delay			(unsigned char);

/*************************************************************************/
/****           Erase a Block of FLASH Memory Subroutine    	   *****/
/*************************************************************************/
/* This routine erases a block of FLASH of the size specified in the 	 */
/* FLASH Control Registers and at the location specified by FLASH_addr.  */
/* FLASH_addr is any address within the block to be erased.  The routine */
/* follows the basic 9 step sequence of the FLASH erase operation.       */
/*									 	  		 */
/* Initializations required:								 */
/*	Set charge pump clock and block erase size in FLCR1 and FLCR2;     */
/*	FLASH_addr set					  				 */
/* Values returned:									 */
/*	none						  					 */
/*************************************************************************/
void EraseRoutine ()
{
	unsigned char Blk_Protect1;	/* set up local variables 		 */
	unsigned char Blk_Protect2;
	unsigned char loop;

	DisableInterrupts;		/* Disable interrupts 			 */

	writeFLCR (ERASE);		/* Step 1 - Set the ERASE bit 	 */

	Blk_Protect1 = FLBPR1;		/* Step 2 - Read from the block 	 */
	Blk_Protect2 = FLBPR2;		/*   protect registers   		 */

	*FLASH_addr = 0xFF;		/* Step 3 - Write to any FLASH 	 */
						/*   address within the block address*/
						/*   range with any data value 	 */
  
	writeFLCR (HVEN);			/* Step 4 - Set the HVEN bit 		 */

	for (loop=0x15; loop !=0; loop--)
	     Delay (tERASE);		/* Step 5 - Wait for time tERASE 	 */

	writeFLCR (HVEN);			/* Step 6 - Clear the HVEN bit  	 */

	Delay (tKILL);			/* Step 7 - Wait for time tKILL 	 */

	writeFLCR (ERASE);		/* Step 8 - Clear the ERASE bit 	 */

	Delay (tHVD);			/* Step 9 - Wait for time tHVD 	 */

	EnableInterrupts;			/* Enable Interrupts 			 */

Erase990:    
	return;
}

/*************************************************************************/
/****      Program/Margin Read 8 Bytes of FLASH Memory Subroutine    *****/
/*************************************************************************/
/* This routine programs the 8 bytes of data from the RAM data array to  */
/* the FLASH memory starting at FLASH_addr. The programming is done in   */
/* ascending order, ie. data[0] -> FLASH_addr[0], data[1] -> 	       */
/* FLASH_addr[1], etc.				 					 */	
/*										 	 	 */
/* The routine is structured into a loop for smart programming so that 	 */
/* the minimum amount of high voltage is applied to the FLASH. Each time */
/* through the loop is called a "pulse".  Each pulse consists of all 17  */
/* steps shown in the application note, with the programming time, tSTEP,*/
/* equal to approximately 1ms. 							 */
/*												 */
/* If all 8 bytes program correctly in the approximately 1ms time period,*/
/* then the routine exits and returns control to the calling program. If */
/* the 8 bytes do not program correctly, another attempt(pulse) is made. */
/* This continues for the maximum number of pulses (flsPULSES). If the 8 */
/* bytes are not programmed successfully after a maximum number of 	 */
/* pulses, then the programming error flag is incremented, the routine is*/
/* exited and the error value (any non-zero value) is returned. 		 */
/*												 */
/*  Initializations required:							   	 */
/*	Set charge pump clock in FLCR1 and FLCR2; FLASH_addr set; data     */
/*	buffer filled								       */  
/*  Values returned:									 */
/*	Program Success/Error Flag						       */
/*************************************************************************/
unsigned char Prog8Bytes ()
{							
	unsigned char Blk_Protect1;	/* set up local variables 		 */
	unsigned char Blk_Protect2;	
	unsigned char byte,status;	

	DisableInterrupts;		/* No interrupts allowed during	 */
						/*   programming 				 */

	attempt=0;				/* Step 1 - Clear attempt counter 	 */

	do {					/* Set up Repeat Loop 			 */

		writeFLCR (PGM);		/* Step 2 - Set the PGM bit	  	 */

		Blk_Protect1 = FLBPR1;	/* Step 3 - Read from the block 	 */	
		Blk_Protect2 = FLBPR2;	/*   protect registers 			 */

						/* Step 4 - Copy the 8 bytes of data */
						/*   from the RAM buffer to the	 */ 
						/*   appropriate FLASH locations 	 */
		for (byte=0;byte<8;byte++)
			*(FLASH_addr+byte)=data[byte];
									
		writeFLCR (HVEN);		/* Step 5 - Set the HVEN bit 		 */

		Delay (tSTEP);		/* Step 6 - Wait for time tSTEP 	 */

		writeFLCR (HVEN);		/* Step 7 - Clear the HVEN bit   	 */

		Delay (tHVTV);		/* Step 8 - Wait for time tHVTV  	 */
	
		status=0;

		writeFLCR (MARGIN);	/* Step 9 - Set the MARGIN bit 	 */

		Delay (tVTP);		/* Step 10 - Wait for time tVTP 	 */

		writeFLCR (PGM);		/* Step 11 - Clear the PGM bit 	 */

		Delay (tHVD);		/* Step 12 - Wait for time tHVD 	 */

		for (byte=0; byte<8; byte++)
			if (*(FLASH_addr+byte) == data[byte]) status++;

						/* Step 13 and 14 - Check to see if	 */
						/*   the correct data was programmed */
						/*   in the FLASH page. If not, 	 */
						/*   proceed with the bracketed 	 */
						/*   expressions. If so, skip to next*/
						/*   if statement. 			 */

		if (status != 8) {
						/* -- Program Failure --		 */
						/* Step 15 - Clear the MARGIN bit 	 */
			writeFLCR (MARGIN);	
			attempt++; 		/* Step 16 - Increment attempt 	 */
						/*	counter 				 */
			}
	
		if (status == 8) 		/* -- Program Success -- 		 */
			writeFLCR (MARGIN);	
						/* Clear the MARGIN bit	             */

	} while ((status!=8) && (attempt<flsPULSES));
						/* Step 17 - If attempt is less than */
						/*   flsPULSES, go back to step 2 	 */

	EnableInterrupts;			/* Clear the interrupt mask bit 	 */

Prog990:
	return (8-status);		/* If programming was unsuccessful in*/
						/*   flsPULSES, a non-zero value is  */
						/*   returned  				 */
}

/*************************************************************************/
/****      			Write to a Flash Control Register            *****/
/*************************************************************************/
/* This routine determines whether flcr1 or flcr2 should be written to   */
/* based on the Flash address specified by FLASH_addr.			 */
/*												 */
/* Initializations required:								 */
/*	set FLASH_addr									 */
/* Values returned:								  	 */
/*	None											 */
/*************************************************************************/
void writeFLCR (unsigned char A)
{
						/* FLASH-1=$8000, FLASH-2=$0450 	 */
	if (FLASH_addr >= &FLASH1)	/* If FLASH_addr is in FLASH-1 array */
		FLCR1 = FLCR1 ^ A; 	/*   write to FLCR1 Register 		 */
      					   
	else 
		FLCR2 = FLCR2 ^ A;	/* else write to FLCR2 Register 	 */

FLCR990:
	return;
}

/*************************************************************************/
/****                        Delay Routine                           *****/
/*************************************************************************/
/* This routine delays for a variable amount of time depending on the 	 */
/* value passed into the routine as variable A.					 */
/* Delay time = (12 + (5 + (11+72+11)*A + 11) + 8) / Bus Freq		 */
/* This equation includes the bus cycles from calling the delay routine  */
/* until hitting the next line of code. This allows the user to  		 */
/* accurately predict the delay between two steps in the code.  		 */
/*												 */
/* Initializations required:							  	 */
/*	Pass in variable A = Delay Value		          			 */
/* Values returned:								       */
/*	None											 */
/*************************************************************************/
void Delay (unsigned char A)
{
	unsigned char	count;

	while (A!=0) {
		for (count=0x03;count!=0;count--);
		A--;
	}

Delay990:
	return;
}


