/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*     Program / Margin Read FLASH 2TS Memory on the MC68HC908AS60       */
/*                                                                	 */
/*************************************************************************/
/* File Name: Prog_mrt.c                   Copyright (c) Motorola 1999   */
/*                                                                       */
/* Current Revision: 1.0                                          	 */
/* Current Release Level: RP                                      	 */
/* Current Revision Release Date:  10/02/99                       	 */
/*                                                                	 */
/* Current Release Written By: Adeela Gill                        	 */
/*                             Motorola Systems Engineering       	 */
/*					 Austin, Texas					 */
/*                                                                	 */
/* Compiled Under: HiCross HC08 (HiWare)   Ver.: 5.0.5  		       */
/*                                                                	 */
/* Project Folder Name: FLASH_2TS                                 	 */
/*                                                                	 */
/* Part Family Software Routine Works With: HC08                         */
/* Part Module(s) Software Routine Works With: fls32k_a01                */
/*                                                                       */
/* Routine Size (Bytes):	368                                 	 */
/* Stack Space Used (Bytes):  12							 */
/* RAM Used (Bytes):		11							 */
/* Global Variables Used:	FLASH_addr, data, attempt                  */
/* Subroutine(s) Called:	Prog8Bytes              	 		 */
/*                                                                       */
/* Full Functional Description Of Routine Design:                        */
/*   Prog_mrt.c is the main routine for the programming operation.  It   */
/*   demonstrates a smart programming algorithm that minimizes the       */
/*   amount of time needed to program a page of 2TS FLASH memory on the  */
/*   MC68HC908AS60.  One page consists of eight consecutive bytes of     */
/*   FLASH memory starting at either address $xxx0 or $xxx8.             */
/*                                                                       */
/*************************************************************************/
/* Motorola reserves the right to make changes without further notice to */
/* any product herein. Motorola makes no warranty, representation or 	 */
/* guarantee regarding the suitability of its products for any particular*/
/* purpose, nor does Motorola assume any liability arising out of the    */
/* application or use of any product, circuit, and specifically disclaims*/
/* any and all liability, including without limitation consequential or	 */ 
/* incidental damages. "Typical" parameters can and do vary in different */
/* applications. All operating parameters, including "Typicals" must be  */
/* validated for each customer application by customer's technical 	 */
/* experts. Motorola does not convey any license under its patent rights */
/* nor the rights of others. Motorola products are not designed, 		 */
/* intended, or authorized for use as components in systems intended for */
/* surgical implant into the body, or other applications intended to 	 */
/* support or sustain life, or for any other application in which the    */
/* failure of the Motorola product could create a situation where 	 */
/* personal injury or death may occur. Should Buyer purchase or use    	 */
/* Motorola products for any such intended or unauthorized application,  */
/* Buyer shall indemnify and hold Motorola and its officers, employees,  */
/* subsidiaries, affiliates, and distributors harmless against all 	 */
/* claims, costs, damages, and expenses, and reasonable attorney fees 	 */
/* arising out of, directly or indirectly, any claim of personal injury  */
/* or death associated with such unintended or unauthorized use, even if */
/* such claim alleges that Motorola was negligent regarding the design or*/
/* manufacture of the part. Motorola and the Motorola symbol are 		 */
/* registered trademarks of Motorola, Inc.					 */
/*************************************************************************/

/*************************************************************************/
/*****                       Include Files		  	  	   *****/
/*************************************************************************/
#include <as60_flash_frk.c>		/* Equates for the registers and bits*/
						/*   of the HC908AS60 that are used	 */ 
#include <prog_er_var.c>		/* RAM variable definitions	 	 */

/*************************************************************************/
/*****                     Function Definitions		  	         *****/
/*************************************************************************/
extern void 		writeFLCR 	(unsigned char);
extern unsigned char	Prog8Bytes 	(void);

/*************************************************************************/
/*****             Program/Margin Read Main Routine                  *****/
/*************************************************************************/
/* This routine initializes the 908AS60 before calling the programming   */
/* routine, Prog8Bytes. If a user plans to incorporate Prog8Bytes into   */
/* his/her program, make sure the FLASH control registers, FLASH_addr 	 */
/* and data buffer are initialized before calling Prog8Bytes. 	       */
/*												 */
/* After the initialization, Prog8Bytes is called to program 8 data 	 */
/* bytes, or one page of FLASH memory.  If programming was successful, 	 */
/* then the program will jump to the while(1) statement in the "if" 	 */
/* expression. If programming failed, then the program will remain at 	 */
/* the while(1) statement in the "else" expression.				 */
/*************************************************************************/
void main ()
{
	unsigned char	count;	/* Variable to count down bits 	 */ 
	unsigned char 	pgmsuccess;	/* Return value from Prog8Bytes 	 */

	CONFIG1 = 0x71;			/*Turn off the COP, but leave the    */
						/*  LVI on					 */

	FLCR1 = 0x00;			/*Set up the frequency divide 	 */
	FLCR2 = 0x00;			/*  control bits in flcr1 & 2 for    */
						/*  the appropriate charge pump 	 */
						/*  clock					 */
	
	FLASH_addr = (unsigned char *)0x8000;
						/*Load FLASH_addr with address of	 */
						/*  where the 8 bytes should start 	 */
						/*  being programmed. Must be XXX0 	 */
						/*  or XXX8					 */

	for (count = 8; count != 0; count--) 
		data[count-1]=count;	/* Fill the RAM array, data, with 	 */
						/*   values to program into FLASH. 	 */
						/*   (ie. 01,02,03,04,05,06,07,08)	 */	

	pgmsuccess=Prog8Bytes();	/* Program the 8 data bytes		 */

	if (pgmsuccess!=0) 		/* Check if a programming error	 */ 
						/*   occurred				 */

		while (1);			/* -- Programming failed --	       */
						/*   Take appropriate action		 */
	
	else while (1);			/* -- Programming successful --	 */
						/*   End of program			 */
}

/*************************************************************************/
/*****               Subroutine Body Includes Section                *****/
/*************************************************************************/
#include	<prog_er_srt.c>		/* Prog8Bytes, WriteFLCR and Delay 	 */
						/*   Subroutines				 */

